/**
 * Clase   que manipula la conexion del lado del Servidor.
 * @author nsanz
 */
import java.io.*;
import javax.swing.text.JTextComponent;
import java.net.*;
import javax.swing.JOptionPane;

public class ConexionServidor extends AbstractConexion {

    private ObjectOutputStream oos;
    private Socket connectionSocket;
    
    /**
     * Constructor de la conexion desde el lado del servidor.
     * @param componenteTexto Componente correspondiente al cuadro de texto de edicion.
     * @param archivo Nombre de archivo.
     */
    public ConexionServidor(JTextComponent componenteTexto, String archivo) {
        super(componenteTexto, archivo);
    }
    /**
     * Inicializa los parametros de la conexion
     * @param ip Direccion IP a la que se desea conectar, null en caso de ser servidor.
     * @param puerto
     */
    public void configurarConexion(String ip, int puerto) {
        super.ip = ip;
        super.puerto = puerto;
    }
    /**
     * Implementacion de Runnable, para la creacion de nuevas hebras para manejar las conexione. Se encarga de inicializar la conexion.
     */
    public void run() {
        try {
            ServerSocket welcomeSocket = new ServerSocket(puerto);
            while (true) {
                connectionSocket = welcomeSocket.accept();

                // Llega un cliente.
                System.out.println("Aceptado cliente");

                // Cuando se cierre el socket, esta opci�n hara que el cierre se
                // retarde autom�ticamente hasta 10 segundos dando tiempo al cliente
                // a leer los datos.
                connectionSocket.setSoLinger(true, 100);

                // Se lee el mensaje de petici�n de fichero del cliente.
                ObjectInputStream ois = new ObjectInputStream(connectionSocket.getInputStream());
                Object mensaje = ois.readObject();

                // Si el mensaje es de petici�n de fichero
                if (mensaje instanceof MensajePide && ((MensajePide) mensaje).opcion == MensajePide.Opciones.PEDIR_INICIAL) {
                    // Se muestra en pantalla el fichero pedido y se envia
                    System.out.println("Me piden el archivo original");
                    enviaFichero(componenteTexto.getText(), new ObjectOutputStream(connectionSocket.getOutputStream()));
                } else {
                    // Si no es el mensaje esperado, se avisa y se sale todo.
                    System.err.println(
                            "Mensaje no esperado " + mensaje.getClass().getName());
                }

            //DataOutputStream  outToClient = new DataOutputStream(connectionSocket.getOutputStream());
            //outToClient.writeBytes(componenteTexto.getText().toString() + componenteTexto.getText());
            }
        } catch (Exception e) {
            JOptionPane.showMessageDialog(
                    componenteTexto, e, "Error en la conexion",
                    JOptionPane.ERROR_MESSAGE);
        }
    }
/**
 * Para finalizar la conexion.
 */
    public void cerrarConexion() {

    }
/**
 * Medoto utilizado para enviar el contenido completo de un String grande. Fracciona este contenido encapsulandolo en un mensaje especial.
 * @param texto String a enviar
 * @param oos   Stream de salida de Objetos.
 */
    private void enviaFichero(String texto, ObjectOutputStream oos) {

        try {
            boolean leidoUltimo = false;

            // Se instancia y rellena un mensaje de envio de fichero
            MensajeTomaFichero mensaje = new MensajeTomaFichero();
            mensaje.nombreFichero = archivo;
            mensaje.opcion = mensaje.opcion.ENVIAR_INICIAL;

            // Se leen los primeros caracteres del documento en un campo del mensaje
            int sBegin = 0;
            int sEnd = MensajeTomaFichero.LONGITUD_MAXIMA;
            if (sEnd >= texto.length()) {
                sEnd = texto.length();
            }
            try {
                mensaje.contenidoLinea = texto.substring(sBegin, sEnd);
            //System.out.println(texto.substring(0, sEnd) + "\n srcBegin,srcEnd " + sBegin + "," + sEnd);
            } catch (Exception e) {
                System.out.println("22222 HAAAAAAAAA fallo leer de la pantalla error: " + e);
            }

            // Bucle mientras se vayan leyendo datos 
            do {
                // Se envia por el socket
                oos.writeObject(mensaje);

                // Se crea un nuevo mensaje
                mensaje = new MensajeTomaFichero();
                mensaje.nombreFichero = archivo;
                mensaje.opcion = MensajeTomaFichero.Opciones.ENVIAR_INICIAL;

                // Se configura un nuevo contenido para enviar
                sBegin += MensajeTomaFichero.LONGITUD_MAXIMA;
                sEnd += MensajeTomaFichero.LONGITUD_MAXIMA;
                if ((sEnd - sBegin) >= (texto.length() - sBegin)) {
                    sEnd = texto.length();
                    leidoUltimo = true;
                } else {
                    leidoUltimo = false;
                }

                try {
                    mensaje.contenidoLinea = texto.substring(sBegin, sEnd);
                } catch (Exception e) {
                    System.out.println("HAAAAAAAAA fallo leer de la pantalla error: " + e);
                    break;
                }

            } while (!leidoUltimo);
            // Encviamos la ultima linea y avisamos que es el ultimo
            mensaje.ultimoMensaje = false;
            oos.writeObject(mensaje);
            this.oos = oos;

        // Se cierra el ObjectOutputStream
        //oos.close();
        } catch (Exception e) {
            e.printStackTrace();
        }


    }
    /**
 * Accion de recivir los caracteres 
 * @param c Caracter que se va a enviar.
 * @param location Posicion en el texto.
 * @param bs Es true si se borra un caracter. Accion de hacer backspace.
 */
    public void recivirCaracteresEditados(char c, int location, boolean bs) {
        System.out.println("Para la version 1.0");
    }
/**
 * Accion de enviar los caracteres 
 * @param c Caracter que se va a enviar.
 * @param location Posicion en el texto.
 * @param bs Es true si se borra un caracter. Accion de hacer backspace.
 */
    public void enviarCaracteresEditados(char c, int location, boolean bs) {
        try {
            // Se instancia y rellena un mensaje
            System.out.println("El caracter editado fue: " + c);
            MensajeTomaCaracter mensaje = new MensajeTomaCaracter();
            mensaje.c = c;
            mensaje.location = location;
            if(c =='\b') mensaje.backSpace=true;
            oos.writeObject(mensaje);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
